/************************************************/
/*												*/
/* IndiaMarket.js								*/
/*												*/
/* Version: 1.0									*/
/* Author:  Kumar S								*/
/*												*/
/* Release Date: 27/11/2007						*/
/* ProjectPage: http://indiamarket.mozdev.org	*/
/* Homepage: http://www.guyfromchennai.com		*/
/*												*/
/* Description :								*/
/*      Get Updates on your desired Stocks from */
/*		Bombay and National exchanges			*/
/*												*/
/************************************************/

/*

Updates
-------
* Added multiple stock lookup
* Exchange can be configured
* Automatic check at preferred time intervals
* Time interval for checking stock values can be configured
* Export stock values configured.
* Import stock values saved elsewhere using indiamarket plugin
* Added the feature to add default stock values on Installation, inorder to avoid empty stock list.
//----------------------------------------------------------------------------------

Bug Fixes
---------
* Removed the multiple headings ontop of every stock while the update process is going on
//----------------------------------------------------------------------------------

Todo
----
* Scroll values in statusbar
* Store Past Values for analysis
* Look options for bulk stocks checking. this will reduce the multiple requests
* url/site to configured ?
* use $,$S functions to reduce code size
* Avoid restarting firefox, after importing/adding new stock values
//----------------------------------------------------------------------------------

*/

//http://content.icicidirect.com/research/topsearchquotes.asp?icicicode=tubinv&category=corporate

function IndiaMarket() {}

IndiaMarket.prototype = {
	isAutomatic : true,
	autocheckInterval: 30,
	stockExchangeStr: "Bse",
	stockCodes : new Array(),
	stockNames : new Array(),
	stockValues : new Array(),
	curStockIndex : 0,
	allStockStr : "",
	myTimer : 0,
	myAllTimer : 0,
	prevSensexVal : 0,
	MaxStocks : 0,
	isAlertable : false
}

IndiaMarket.prototype.init = function() {
	//autocheck enabled ?
	this.isAutomatic = this.getBoolPref("indiamarket.autocheck", true);

	//auto check time interval
	var timeinterval = parseInt(this.getStrPref("indiamarket.autocheckInterval", this.autocheckInterval),10)
	this.autocheckInterval = (timeinterval > 30) ? timeinterval: 30 ;

	//exchange details. whether BSE or NSE
	this.stockExchangeStr = this.getStrPref("indiamarket.stockExchange",this.stockExchangeStr);

	//stocks list
	var list = this.getPrefFilterList();
	for (var i = 0; i < list.length; i++) {
		if (list[i] != "") {
			//alert(list[i]);
			var item = list[i].split("|");
			var sName = item[0];
			var sCode = item[1];
			if (!/^\/(.*)\/$/.exec(sName)) sName = sName.replace(/\/$/, "/*");
			if (!/^\/(.*)\/$/.exec(sCode)) sCode = sCode.replace(/\/$/, "/*");
			var enabled = (item[2] == "TRUE") ? true : false;

			if(enabled){ //only enabled stocks
			 this.stockNames[this.stockNames.length] = sName;
			 this.stockCodes[this.stockCodes.length] = sCode;
			}
			//this.addStock(sName, sCode,enabled);
		}
	}

	if( (this.stockNames.length>0) && (this.stockCodes.length >0) ){
		this.curStockIndex = 0;
		this.MaxStocks = this.stockNames.length;
	}

      document.getElementById("lblHeader").value = "India Market";
      document.getElementById("lblHeader").style.fontFamily = "Verdana, Arial;"
      document.getElementById("lblHeader").style.fontSize = "12px;"
      document.getElementById("lblHeader").style.fontWeight = "bold;"
      document.getElementById("lblHeader").style.margin = "5px;"


	this.refreshInformation();
}

// Called periodically to refresh the stock information
IndiaMarket.prototype.refreshInformation = function(){
	var ret=0;

	//curExchange = this.stockExchangeStr;
	this.StockValues = null;
	this.contactServer();
}

IndiaMarket.prototype.contactServer = function(){
	if(this.stockCodes[this.curStockIndex] == null)
		return false;

	var fullUrl = "http://www.sharekhan.com/News/CompanyBasicQuote.aspx" + "?sskicode=" + this.stockCodes[this.curStockIndex]; //SENSEX

	var tmpStockVal = "";

	this.mRequests(fullUrl,'pStocks',function(req){
		var output = req.responseText;
		var retStatus = req.status;

		if(retStatus == 200){ //success
			//alert(output);
			if (output.length){
				var samplePanel = document.getElementById('indiamarket');
				if(pIndiaMarket.stockExchangeStr=="Bse"){
					var re = new RegExp("<span id=\"BasicQuote1_lblBse1\">([0-9.]*)</span>");
					var m = re.exec(output);

					//prev close
					var pre = new RegExp("<span id=\"BasicQuote1_lblBse2\">([0-9.]*)</span>");
					var pm = pre.exec(output);

					//volume
					var vol = new RegExp("<span id=\"BasicQuote1_lblBse4\">([0-9.]*)</span>");
					var volume = vol.exec(output);

				}
				else if(pIndiaMarket.stockExchangeStr=="Nse"){
					var re = new RegExp("<span id=\"BasicQuote1_lblNse1\">([0-9.]*)</span>");
					var m = re.exec(output);

					//prev close
					var pre = new RegExp("<span id=\"BasicQuote1_lblNse2\">([0-9.]*)</span>");
					var pm = pre.exec(output);

					//volume
					var vol = new RegExp("<span id=\"BasicQuote1_lblNse4\">([0-9.]*)</span>");
					var volume = vol.exec(output);
				}

				if (m == null) {
				}
				else {
					  for (i = 0; i < m.length; i++) {
					    s = m[i];
					  }
					  samplePanel.label = pIndiaMarket.stockCodes[pIndiaMarket.curStockIndex] + ": " + s;

					  this.prevSensexVal = s;

					  tmpStockVal = ""; //reset string
					  tmpStockVal = s; //pIndiaMarket.stockCodes[pIndiaMarket.curStockIndex] + " : " + s ;
				}

				//Previous value
				if (pm == null) {
					//alert('pm is null');
				}
				else {
				  for (i = 0; i < pm.length; i++) {
				    s = pm[i];
				  }
				  pIndiaMarket.stockValues[pIndiaMarket.curStockIndex] = tmpStockVal + " - "+ s ;
				}

				//volume
				if (volume == null) {
					//alert('pm is null');
				}
				else {
				  for (i = 0; i < volume.length; i++) {
				    s = volume[i];
				  }
				  pIndiaMarket.stockValues[pIndiaMarket.curStockIndex] += " - "+ s ;
				}

				pIndiaMarket.status=retStatus;


				var parent = document.getElementById("im-toolbar-tooltip-grid");
				var parentrows = document.getElementById("im-toolbar-tooltip-rows");
				 if(pIndiaMarket.curStockIndex == 0){
				    while (parentrows.hasChildNodes())
						parentrows.removeChild(parentrows.lastChild);

					//heading
					pIndiaMarket.generateHeadings();

				}

				pIndiaMarket.curStockIndex++;

				//clears the time interval
				if (pIndiaMarket.myAllTimer>0){
					clearTimeout(pIndiaMarket.myAllTimer);
				}
				if (pIndiaMarket.myTimer>0)
				{
					clearTimeout(pIndiaMarket.myTimer);
				}

				if(pIndiaMarket.curStockIndex < pIndiaMarket.MaxStocks){
/* 					//heading
					pIndiaMarket.generateHeadings();
 */
					//datarow
					var itemrow = null;
					itemrow = document.createElement("row");

					var item = null;
					item = document.createElement("label");
					item.setAttribute("value", pIndiaMarket.stockCodes[pIndiaMarket.curStockIndex-1]);
					item.setAttribute("style", "width:60px;font-weight:normal");
					itemrow.appendChild(item);

					var item = null;
					item = document.createElement("label");
					item.setAttribute("value", " ");
					item.setAttribute("style", "width:5px;");
					itemrow.appendChild(item);

					var item = null;
					item = document.createElement("label");
					item.setAttribute("value", pIndiaMarket.stockValues[pIndiaMarket.curStockIndex-1].split(" - ")[0]);
					item.setAttribute("style", "width:60px;font-weight:normal;text-align:right");
					itemrow.appendChild(item);

					var item = null;
					item = document.createElement("label");
					item.setAttribute("value", " ");
					item.setAttribute("style", "width:5px;");
					itemrow.appendChild(item);

					var item = null;
					item = document.createElement("label");
					item.setAttribute("value", pIndiaMarket.stockValues[pIndiaMarket.curStockIndex-1].split(" - ")[1]);
					item.setAttribute("style", "width:60px;font-weight:normal;text-align:right");
					itemrow.appendChild(item);

					var item = null;
					item = document.createElement("label");
					item.setAttribute("value", " ");
					item.setAttribute("style", "width:5px;");
					itemrow.appendChild(item);

					var item = null;
					item = document.createElement("label");
					item.setAttribute("value", pIndiaMarket.stockValues[pIndiaMarket.curStockIndex-1].split(" - ")[2]);
					item.setAttribute("style", "width:60px;font-weight:normal;text-align:right");
					itemrow.appendChild(item);


					parentrows.appendChild(itemrow);
					parent.appendChild(parentrows);


					pIndiaMarket.setStatus("Loading Stock Details of ..."+pIndiaMarket.stockNames[pIndiaMarket.curStockIndex]);

					pIndiaMarket.myTimer = setTimeout('pIndiaMarket.refreshInformation();',10000);
					//pIndiaMarket.refreshInformation();
				}else{
					pIndiaMarket.curStockIndex=0;
					while (parentrows.hasChildNodes())
						parentrows.removeChild(parentrows.lastChild);

					//heading
					if(pIndiaMarket.stockValues.length > 0){
						pIndiaMarket.generateHeadings();
					}

					for (var ctr = 0; ctr < pIndiaMarket.stockValues.length; ctr++)
					{
						//datarow
						var itemrow = null;
						itemrow = document.createElement("row");

						var item = null;
						item = document.createElement("label");
						item.setAttribute("value", pIndiaMarket.stockCodes[ctr]);
						item.setAttribute("style", "width:60px;font-weight:normal");
						itemrow.appendChild(item);

						var item = null;
						item = document.createElement("label");
						item.setAttribute("value", " ");
						item.setAttribute("style", "width:5px;");
						itemrow.appendChild(item);

						var item = null;
						item = document.createElement("label");
						item.setAttribute("value", pIndiaMarket.stockValues[ctr].split(" - ")[0]);
						item.setAttribute("style", "width:60px;font-weight:normal;text-align:right");
						itemrow.appendChild(item);

						var item = null;
						item = document.createElement("label");
						item.setAttribute("value", " ");
						item.setAttribute("style", "width:5px;");
						itemrow.appendChild(item);

						var item = null;
						item = document.createElement("label");
						item.setAttribute("value", pIndiaMarket.stockValues[ctr].split(" - ")[1]);
						item.setAttribute("style", "width:60px;font-weight:normal;text-align:right");
						itemrow.appendChild(item);

						var item = null;
						item = document.createElement("label");
						item.setAttribute("value", " ");
						item.setAttribute("style", "width:5px;");
						itemrow.appendChild(item);

						var item = null;
						item = document.createElement("label");
						item.setAttribute("value", pIndiaMarket.stockValues[ctr].split(" - ")[2]);
						item.setAttribute("style", "width:80px;font-weight:normal;text-align:right");
						itemrow.appendChild(item);

						parentrows.appendChild(itemrow);
						parent.appendChild(parentrows);
					}

					pIndiaMarket.setStatus();

					//if(pIndiaMarket.isAlertable)
					//	alert("Stocks and Values"+"\n"+pIndiaMarket.allStockStr);

					//if auto check is enabled then auto check in the set time intervals
					if(pIndiaMarket.isAutomatic)
						pIndiaMarket.myAllTimer = setTimeout('pIndiaMarket.refreshInformation();',((parseInt(pIndiaMarket.autocheckInterval,10))*60*1000) );

					//pIndiaMarket.refreshInformation();
				}
			}//end if output
		} //end if success
		/*
		else if(retStatus == 403){ //forbidden
			var samplePanel = document.getElementById('indiamarket');
			samplePanel.label ="Oops!";
			pIndiaMarket.setStatus("Network Error. Please retry after sometime");
		}
		else{
			var samplePanel = document.getElementById('indiamarket');
			samplePanel.label ="Oops!";
			pIndiaMarket.setStatus("Server unavailable. Please retry after sometime");
		}*/
	});

}
IndiaMarket.prototype.generateHeadings = function(){
	var parent = document.getElementById("im-toolbar-tooltip-grid");
	var parentrows = document.getElementById("im-toolbar-tooltip-rows");

	var itemrow = null;
	itemrow = document.createElement("row");


	var item = null;
	item = document.createElement("label");
	item.setAttribute("value", "Stock");
	item.setAttribute("style", "color:#646464;width:60px;font-weight:bold");
	itemrow.appendChild(item);

	var item = null;
	item = document.createElement("label");
	item.setAttribute("value", " ");
	item.setAttribute("style", "width:5px;");
	itemrow.appendChild(item);

	var item = null;
	item = document.createElement("label");
	item.setAttribute("value", "Current");
	item.setAttribute("style", "color:#646464;width:60px;font-weight:bold");
	itemrow.appendChild(item);

	var item = null;
	item = document.createElement("label");
	item.setAttribute("value", " ");
	item.setAttribute("style", "width:5px;");
	itemrow.appendChild(item);

	var item = null;
	item = document.createElement("label");
	item.setAttribute("value", "Prev Close");
	item.setAttribute("style", "color:#646464;width:60px;font-weight:bold");
	itemrow.appendChild(item);

	var item = null;
	item = document.createElement("label");
	item.setAttribute("value", " ");
	item.setAttribute("style", "width:5px;");
	itemrow.appendChild(item);

	var item = null;
	item = document.createElement("label");
	item.setAttribute("value", "Volume");
	item.setAttribute("style", "color:#646464;width:80px;font-weight:bold");
	itemrow.appendChild(item);


	parentrows.appendChild(itemrow);
	parent.appendChild(parentrows);
}

IndiaMarket.prototype.setStatus = function(theStatus){
	if(!theStatus)
		document.getElementById("im-toolbar-tooltip-status-tip1").value = "";
	else
		document.getElementById("im-toolbar-tooltip-status-tip1").value = theStatus;
}
IndiaMarket.prototype.mRequests = function(theUrl,groupName,fnFragment) {
	//alert('mRequests');
  AjaxRequest.get(
    {
      'url':theUrl
      ,'onLoading':function(req){
	      	var samplePanel = document.getElementById('indiamarket');
      		samplePanel.label = "Loading...";
      		//samplePanel.tooltipText ="Fetching Data";
      }
      ,'onComplete':function(req){
	      	var samplePanel = document.getElementById('indiamarket');
      		samplePanel.label = "Completed";
      		//samplePanel.tooltipText ="Completed";
      }
      ,'onSuccess':function(req){
	       if(fnFragment){
	        	fnFragment(req);
	       }
	      }
      ,'onError':function(req){
	      this.AjaxRequestEnd(false);
	      }
      ,'groupName':groupName                    // Assigns this request to the group passed in
      ,'onGroupBegin':this.AjaxRequestBegin()     // Map to the function called when this group starts
      ,'onGroupEnd':this.AjaxRequestEnd(false)     // Map to the function called when this group ends
      ,'timeout':60000	//timeout is 1 min
      ,'onTimeout':function(req){
	      	var samplePanel = document.getElementById('indiamarket');
      		samplePanel.label = "Timed Out";
      		//samplePanel.tooltipText = "The update server seems to be unavailable. Update will be attempted again later";
      }
    }
  );
}
IndiaMarket.prototype.AjaxRequestBegin = function(){
	var samplePanel = document.getElementById('indiamarket');
	samplePanel.label = "Updating...";
	//samplePanel.tooltipText ="Attempting update of data...";
}
IndiaMarket.prototype.AjaxRequestEnd = function(theStatus){
	var samplePanel = document.getElementById('indiamarket');
	if(theStatus==true){
		samplePanel.label = "Success";
		//samplePanel.tooltipText ="Successfully Got Data";
	}
	else{
		samplePanel.label ="Oops!";
		//samplePanel.tooltipText = "The update server seems to be unavailable. Update will be attempted again later";
	}
}


IndiaMarket.prototype.getPrefFilterList = function() {
   var s = this.getStrPref("indiamarket.filterlist", null);
   return (s ? s.split("|*|") : "");
}

//-----------------------------------------------------------------------------
//Function to trim the space in the  string
IndiaMarket.prototype.trim = function(s) {
   if (s) return s.replace(/^\s+/g,"").replace(/\s+$/g,""); else return "";
}
//Function to trim the space in the  string
IndiaMarket.prototype.startsWith = function(s, prefix) {
   if (s) return( (s.substring(0, prefix.length) == prefix) ); else return false;
}


//-----------------------------------------------------------------------------

IndiaMarket.prototype.getBoolPref = function(prefName, defval) {
   var result = defval;
   var prefservice = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
   var prefs = prefservice.getBranch("");
   if (prefs.getPrefType(prefName) == prefs.PREF_BOOL) {
       try { result = prefs.getBoolPref(prefName); }catch(e){}
   }
   return(result);
}

IndiaMarket.prototype.getIntPref = function(prefName, defval) {
   var result = defval;
   var prefservice = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
   var prefs = prefservice.getBranch("");
   if (prefs.getPrefType(prefName) == prefs.PREF_INT) {
       try { result = prefs.getIntPref(prefName); }catch(e){}
   }
   return(result);
}

IndiaMarket.prototype.getStrPref = function(prefName, defval) {
   var result = defval;
   var prefservice = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
   var prefs = prefservice.getBranch("");
   if (prefs.getPrefType(prefName) == prefs.PREF_STRING) {
       try { result = prefs.getComplexValue(prefName, Components.interfaces.nsISupportsString).data; }catch(e){}
   }
   return(result);
}


// Install load handler
pIndiaMarket = new IndiaMarket();

window.addEventListener("load", function(e) { pIndiaMarket.init(); }, false);
